@@ -1,5 +1,6 @@
 package bin;
 
+import java.io.File;
 import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.HashMap;
@@ -25,6 +26,8 @@ import stream.SamLineStreamer;
 import stream.SamReadInputStream;
 import stream.SamStreamer;
 import structures.ByteBuilder;
+import structures.FloatList;
+import structures.IntList;
 import structures.IntLongHashMap;
 import structures.ListNum;
 import tax.TaxTree;
@@ -88,7 +91,7 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 		samFilter.includeDuplicate=false;
 		samFilter.includeNonPrimary=false;
 		samFilter.includeQfail=false;
-		samFilter.minMapq=4;
+//		samFilter.minMapq=4;
 		
 		{//Parse the arguments
 			final Parser parser=parse(args);
@@ -99,7 +102,6 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 			overwrite=ReadStats.overwrite=parser.overwrite;
 			append=ReadStats.append=parser.append;
 			
-			in=parser.in1;
 			extin=parser.extin;
 
 			out=parser.out1;
@@ -119,11 +121,9 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 		checkStatics(); //Adjust file-related static fields as needed for this program 
 		
 		//Create output FileFormat objects
-		ffout=FileFormat.testOutput(out, FileFormat.SAM, extout, true, overwrite, append, ordered);
-		assert(false) : "TODO: Default output format might be fasta.";
+		ffout=FileFormat.testOutput(out, FileFormat.FASTA, extout, true, overwrite, append, ordered);
 
 		//Create input FileFormat objects
-		ffin=FileFormat.testInput(in, FileFormat.SAM, extin, true, true);
 		ffref=FileFormat.testInput(ref, FileFormat.FASTA, null, true, true);
 	}
 	
@@ -152,10 +152,18 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 			
 			if(a.equals("verbose")){
 				verbose=Parse.parseBoolean(b);
-			}else if(a.equals("ref")){
+			}else if(a.equals("ref") || a.equals("contigs") || a.equals("in")){
 				ref=b;
 			}else if(a.equals("ordered")){
 				ordered=Parse.parseBoolean(b);
+			}else if(a.equals("delimiter")){
+				delimiter=Parse.parseSymbol(b);
+			}else if(a.equals("wipe")){
+				wipe=Parse.parseBoolean(b);
+			}else if(a.equals("depth") || a.equals("cov") || a.equals("adddepth")){
+				addDepth=Parse.parseBoolean(b);
+			}else if(a.equals("tid") || a.equalsIgnoreCase("taxid")){
+				addTid=Parse.parseBoolean(b);
 			}else if(a.equals("clearfilters")){
 				if(Parse.parseBoolean(b)){
 					samFilter.clear();
@@ -163,8 +171,12 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 			}else if(a.equals("parse_flag_goes_here")){
 				long fake_variable=Parse.parseKMG(b);
 				//Set a variable here
+			}else if(a.equals("sam") || a.equals("insam") || a.equals("samin")){
+				Tools.getFileOrFiles(b, in, false, false, true, false);
 			}else if(samFilter.parse(arg, a, b)){
 				//do nothing
+			}else if(new File(arg).isFile()){
+				in.add(arg);
 			}else if(parser.parse(arg, a, b)){//Parse standard flags in the parser
 				//do nothing
 			}else{
@@ -189,7 +201,6 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 		if(in==null){throw new RuntimeException("Error - an input file is required.");}
 
 		//Ensure there is an input file
-		assert(false) : "TODO: Check.";
 		if(ref==null){throw new RuntimeException("Error - a reference file is required.");}
 		
 		//Ensure output files can be written
@@ -198,13 +209,19 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 			throw new RuntimeException("\n\noverwrite="+overwrite+"; Can't write to output file "+out+"\n");
 		}
 		
+		ArrayList<String> x=new ArrayList<String>();
+		x.addAll(in);
+		x.add(ref);
+		
 		//Ensure input files can be read
-		if(!Tools.testInputFiles(false, true, in, ref)){
+		if(!Tools.testInputFiles(false, true, x)){
 			throw new RuntimeException("\nCan't read some input files.\n");  
 		}
 		
+		x.add(out);
+		
 		//Ensure that no file was specified multiple times
-		if(!Tools.testForDuplicateFiles(true, in, ref, out)){
+		if(!Tools.testForDuplicateFiles(true, x.toArray(new String[0]))){
 			throw new RuntimeException("\nSome file names were specified multiple times.\n");
 		}
 	}
@@ -222,7 +239,7 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 	/** Ensure parameter ranges are within bounds and required parameters are set */
 	private boolean validateParams(){
 //		assert(minfoo>0 && minfoo<=maxfoo) : minfoo+", "+maxfoo;
-		assert(false) : "TODO";
+//		assert(false) : "TODO";
 		return true;
 	}
 	
@@ -237,22 +254,25 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 		final boolean vic=Read.VALIDATE_IN_CONSTRUCTOR;
 		Read.VALIDATE_IN_CONSTRUCTOR=Shared.threads()<4;
 		
-		//Create a read input stream
-		final SamStreamer ss=makeStreamer(ffin);
-		
-		scafMap=makeScafMap();
-		
-		//Optionally create a read output stream
-		final ConcurrentReadOutputStream ros=makeCros();
-		
 		//Reset counters
 		readsProcessed=readsOut=0;
 		basesProcessed=basesOut=0;
-		
-		//Process the reads in separate threads
-		spawnThreads(ss, ros);
-		//Close the read streams
-		errorState|=ReadWrite.closeStream(ros);
+		for(int i=0; i<in.size(); i++) {
+			FileFormat ffin=FileFormat.testInput(in.get(i), FileFormat.SAM, extin, true, true);
+
+			//Create a read input stream
+			SamStreamer ss=makeStreamer(ffin);
+
+			if(scafMap.isEmpty()) {scafMap=makeScafMap();}
+
+			//Process the reads in separate threads
+			spawnThreads(ss);
+			
+			for(Scaf scaf : scafMap.values()) {
+				scaf.process();
+				scaf.map.clear();
+			}
+		}
 		
 		processReference();
 		
@@ -300,7 +320,7 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 		for(byte[] line : header) {
 			if(Tools.startsWith(line, "@SQ\t")){
 				lp.set(line);
-				assert(lp.termEquals("SN", 1));
+				assert(lp.termEquals("SN", 1)) : "\n"+lp.parseString(1)+"\n"+lp+"\n"+new String(line);
 				String name=lp.parseString(2);
 				assert(lp.termEquals("LN", 3));
 				int len=lp.parseInt(4);
@@ -328,7 +348,7 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 	/*--------------------------------------------------------------*/
 	
 	/** Spawn process threads */
-	private void spawnThreads(final SamStreamer ss, final ConcurrentReadOutputStream ros){
+	private void spawnThreads(final SamStreamer ss){
 		
 		//Do anything necessary prior to processing
 		
@@ -338,7 +358,7 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 		//Fill a list with ProcessThreads
 		ArrayList<ProcessThread> alpt=new ArrayList<ProcessThread>(threads);
 		for(int i=0; i<threads; i++){
-			alpt.add(new ProcessThread(ss, ros, i));
+			alpt.add(new ProcessThread(ss, i));
 		}
 		
 		//Start the threads
@@ -374,7 +394,7 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 		ListNum<Read> ln=null;
 		for(ln=cris.nextList(); ln!=null && ln.size()>0; ln=cris.nextList()) {
 			for(Read r : ln) {
-				r.id=(rename(r.id, r.length()));
+				r.id=(rename(r.id, r.length(), r.numericID));
 				readsOut++;
 				basesOut+=r.length();
 			}
@@ -392,46 +412,50 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 		errorState|=ReadWrite.closeStreams(cris, ros);
 	}
 	
-	private static String toShortName(String s) {
+	static String toShortName(String s) {
 		int idx=Tools.indexOfWhitespace(s);
 		return idx<0 ? s : s.substring(0, idx);
 	}
 	
-	private String rename(final String old, final int scaflen) {
+	static String toShortName(byte[] s) {
+		int idx=Tools.indexOfWhitespace(s);
+		return idx<0 ? new String(s) : new String(s, 0, idx);
+	}
+	
+	private String rename(final String old, final int scaflen, long numericID) {
 		final String name=old;
 		final String shortName=toShortName(name);
 		final int oldTaxid=TaxTree.parseHeaderStatic2(old, null);
 		Scaf scaf=scafMap.get(shortName);
 		if(scaf==null) {scaf=scafMap.get(name);}
 		assert(scaf!=null) : "Can't find "+shortName+" in map of size "+scafMap.size();
-		if(scaf==null) {return old+" cov_0";}
+		if(scaf==null) {return old+delimiter+"cov_0";}
 		
-		final int[] keys=scaf.map.keys();
-		final long[] values=scaf.map.values();
-		final int invalid=scaf.map.invalid();
-		int taxid=-1;
-		long totalMappedBases=0;
-		long maxMappedBases=0;
-		for(int i=0; i<keys.length; i++) {
-			int key=keys[i];
-			long value=values[i];
-			if(key!=invalid) {
-				totalMappedBases+=value;
-				if(value>maxMappedBases) {
-					maxMappedBases=value;
-					taxid=key;
-				}
+		IntList tids=scaf.tidList;
+		FloatList covs=scaf.covList;
+		float maxCov=0;
+		int maxTid=-1;
+		for(int i=0; i<covs.size(); i++) {
+			int tid=tids.get(i);
+			float cov=covs.get(i);
+			if(maxTid<=0 || cov>maxCov) {
+				maxCov=cov;
+				maxTid=tid;
 			}
 		}
-		final int len=Tools.max(scaflen, scaf.length);
-		assert(len>0);
-		assert(scaflen==scaf.length);
-		final float cov=(float)(maxMappedBases/len);
 		
-		ByteBuilder bb=new ByteBuilder(old.length()+40);
-		bb.append(old);
-		if(oldTaxid<0 && taxid>0) {bb.append(" tid_").append(taxid);}
-		bb.append(" cov_").append(cov, 2);
+		ByteBuilder bb=new ByteBuilder(old.length()+(covs.size*9)+20);
+		if(wipe) {
+			bb.append("contig_").append(numericID);
+		}else {
+			bb.append(old);
+		}
+		
+		if(oldTaxid<0 && maxTid>0 && addTid) {bb.append(delimiter).append("tid_").append(maxTid);}
+		for(int i=0; i<covs.size && addDepth; i++) {
+			bb.append(delimiter).append("cov_").append(covs.get(i), 2);
+		}
+//		assert(false) : oldTaxid+", "+maxTid+", "+bb;
 		return bb.toString();
 	}
 	
@@ -444,7 +468,7 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 	class ProcessThread extends Thread {
 		
 		//Constructor
-		ProcessThread(final SamStreamer ss_, final ConcurrentReadOutputStream ros_, final int tid_){
+		ProcessThread(final SamStreamer ss_, final int tid_){
 			ss=ss_;
 			tid=tid_;
 		}
@@ -530,10 +554,37 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 			synchronized(this) {map.increment(taxid, length);}
 		}
 		
+		void process() {
+			
+			final int[] keys=map.keys();
+			final long[] values=map.values();
+			final int invalid=map.invalid();
+			int taxid=-1;
+			long totalMappedBases=0;
+			long maxMappedBases=0;
+			for(int i=0; i<keys.length; i++) {
+				int key=keys[i];
+				long value=values[i];
+				if(key!=invalid) {
+					totalMappedBases+=value;
+					if(value>maxMappedBases) {
+						maxMappedBases=value;
+						taxid=key;
+					}
+				}
+			}
+			assert(length>0);
+			final float cov=totalMappedBases/(float)length;
+			tidList.add(taxid);
+			covList.add(cov);
+		}
+		
 		final String name;
 		final int length;
 		
 		IntLongHashMap map=new IntLongHashMap(7);
+		IntList tidList=new IntList(2);
+		FloatList covList=new FloatList(2);
 		
 	}
 	
@@ -541,9 +592,9 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 	/*----------------            Fields            ----------------*/
 	/*--------------------------------------------------------------*/
 
-	/** Primary input file path */
-	private String in=null;
-	/** Secondary input file path */
+	/** Sam input file path */
+	private ArrayList<String> in=new ArrayList<String>();
+	/** Contig input file path */
 	private String ref=null;
 
 	/** Primary output file path */
@@ -568,6 +619,11 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 
 	/** Quit after processing this many input reads; -1 means no limit */
 	private long maxReads=-1;
+
+	private String delimiter=" ";
+	private boolean wipe=false;
+	private boolean addDepth=true;
+	private boolean addTid=true;
 	
 	/*--------------------------------------------------------------*/
 	
@@ -585,9 +641,7 @@ public class ContigRenamer implements Accumulator<ContigRenamer.ProcessThread> {
 	/*--------------------------------------------------------------*/
 	/*----------------         Final Fields         ----------------*/
 	/*--------------------------------------------------------------*/
-
-	/** Primary input file */
-	private final FileFormat ffin;
+	
 	/** Secondary input file */
 	private final FileFormat ffref;
 	
