@@ -1,6 +1,5 @@
 package bin;
 
-import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.concurrent.locks.ReadWriteLock;
@@ -20,7 +19,10 @@ import stream.ConcurrentReadInputStream;
 import stream.ConcurrentReadOutputStream;
 import stream.Read;
 import structures.ByteBuilder;
+import structures.IntHashMap;
+import structures.IntLongHashMap;
 import structures.ListNum;
+import tax.TaxTree;
 import template.Accumulator;
 import template.ThreadWaiter;
 import tracker.ReadStats;
@@ -53,7 +55,7 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 		x.process(t);
 		
 		//Close the print stream if it was redirected
-		Shared.closeStream(x.outstream);
+		Shared.closeStream(BinObject.outstream);
 	}
 	
 	/**
@@ -80,9 +82,6 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 			
 			overwrite=ReadStats.overwrite=parser.overwrite;
 			append=ReadStats.append=parser.append;
-			
-			loader.in1=parser.in1;
-			loader.in2=parser.in2;
 
 			outPattern=parser.out1;
 			extout=parser.extout;
@@ -92,7 +91,7 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 		checkFileExistence(); //Ensure files can be read and written
 		loader.checkInput();
 		
-		binner.tree=loader.loadTree();
+		BinObject.tree=(sketchContigs || sketchClusters) ? loader.loadTree() : null;
 		sketcher=(sketchContigs || sketchClusters || sketchOutput) ? new BinSketcher(16, 2000) : null;
 		binner.sketcher=loader.sketcher=sketcher;
 		Sketch.defaultParams.format=DisplayParams.FORMAT_JSON;
@@ -104,6 +103,7 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 	
 	/** Parse arguments from the command line */
 	private Parser parse(String[] args){
+		originalArgs=args.clone();
 		
 		//Create a parser object
 		Parser parser=new Parser();
@@ -125,12 +125,28 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 				verbose=Parse.parseBoolean(b);
 			}
 			
+			else if(a.equals("vectorout")){
+				vectorOut=b;
+			}
+			
 			else if(a.equals("clusterbytax") || a.equals("clusterbytaxid")){
 				clusterByTaxid=Parse.parseBoolean(b);
 			}else if(a.equals("clusterbytet") || a.equals("clusterbytetramer")){
 				clusterByTetramer=Parse.parseBoolean(b);
 			}else if(a.equals("refine") || a.equals("refineclusters")){
 				refineClusters=Parse.parseBoolean(b);
+			}else if(a.equals("residue") || a.equals("processresidue")){
+				processResidue=Parse.parseBoolean(b);
+			}else if(a.equals("purify") || a.equalsIgnoreCase("purifyClusters")){
+				purifyClusters=Parse.parseBoolean(b);
+			}else if(a.equals("entropy") || a.equals("calcentropy")){
+				SpectraCounter.calcEntropy=Parse.parseBoolean(b);
+			}else if(a.equals("strandedness") || a.equals("calcstrandedness")){
+				SpectraCounter.calcStrandedness=Parse.parseBoolean(b);
+			}else if(a.equals("entropyk")){
+				SpectraCounter.entropyK=Integer.parseInt(b);
+			}else if(a.equals("entropywindow")){
+				SpectraCounter.entropyWindow=Integer.parseInt(b);
 			}
 			
 			else if(a.equalsIgnoreCase("sketchcontigs")){
@@ -154,15 +170,61 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 				BinObject.setQuant(Tools.max(1, Integer.parseInt(b)));
 			}
 			
+			else if(a.equals("mincluster") || a.equals("minclustersize")){
+				minBasesPerCluster=Parse.parseIntKMG(b);
+			}else if(a.equals("mincontigs")){
+				minContigsPerCluster=Parse.parseIntKMG(b);
+			}else if(a.equals("validate") || a.equals("validation")){
+				validation=Parse.parseBoolean(b);
+			}else if(a.equalsIgnoreCase("followEdges1") || a.equals("e1")){
+				followEdge1Passes=(Tools.startsWithDigit(b) ? Integer.parseInt(b) :
+					Parse.parseBoolean(b) ? 4 : 0);
+			}else if(a.equalsIgnoreCase("followEdges2") || a.equals("e2") || a.equals("followEdges")){
+				followEdge2Passes=(Tools.startsWithDigit(b) ? Integer.parseInt(b) :
+					Parse.parseBoolean(b) ? 4 : 0);
+			}else if(a.equalsIgnoreCase("edgestringency1")){
+				edgeStringency1=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("edgestringency2") || a.equalsIgnoreCase("edgestringency")){
+				edgeStringency2=Float.parseFloat(b);
+			}
+			
+			else if(a.equalsIgnoreCase("xstrict")){
+				strictnessMult=(Parse.parseBoolean(b) ? 0.6f : 1);
+			}else if(a.equalsIgnoreCase("ustrict")){
+				strictnessMult=(Parse.parseBoolean(b) ? 0.7f : 1);
+			}else if(a.equalsIgnoreCase("vstrict")){
+				strictnessMult=(Parse.parseBoolean(b) ? 0.8f : 1);
+			}else if(a.equalsIgnoreCase("strict")){
+				strictnessMult=(Parse.parseBoolean(b) ? 0.9f : 1);
+			}else if(a.equalsIgnoreCase("normal")){
+				strictnessMult=(Parse.parseBoolean(b) ? 1 : 1);
+			}else if(a.equalsIgnoreCase("loose") || a.equalsIgnoreCase("sensitive")){
+				strictnessMult=(Parse.parseBoolean(b) ? 1.1f : 1);
+			}else if(a.equalsIgnoreCase("vloose") || a.equalsIgnoreCase("vsensitive")){
+				strictnessMult=(Parse.parseBoolean(b) ? 1.2f : 1);
+			}else if(a.equalsIgnoreCase("uloose") || a.equalsIgnoreCase("usensitive")){
+				strictnessMult=(Parse.parseBoolean(b) ? 1.3f : 1);
+			}else if(a.equalsIgnoreCase("xloose") || a.equalsIgnoreCase("xsensitive")){
+				strictnessMult=(Parse.parseBoolean(b) ? 1.4f : 1);
+			}else if(a.equalsIgnoreCase("strictness")){
+				strictnessMult=Float.parseFloat(b);
+			}
+			
+			else if(a.equals("covout") || a.equals("outcov")) {
+				covOut=b;
+			}
+			
 			else if(a.equals("parse_flag_goes_here")){
 				long fake_variable=Parse.parseKMG(b);
 				//Set a variable here
 			}else if(binner.parse(arg, a, b)){
 				//do nothing
-			}else if(loader.parse(arg, a, b)){
+			}else if(Key.parse(arg, a, b)){
 				//do nothing
 			}else if(SimilarityMeasures.parse(arg, a, b)){
 				//do nothing
+			}else if(loader.parse(arg, a, b)){
+				//do nothing
 			}else if(parser.parse(arg, a, b)){//Parse standard flags in the parser
 				//do nothing
 			}else{
@@ -174,6 +236,18 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 		return parser;
 	}
 	
+	private void reprocessArgs() {
+		for(int i=0; i<originalArgs.length; i++){
+			String arg=originalArgs[i];
+			
+			//Break arguments into their constituent parts, in the form of "a=b"
+			String[] split=arg.split("=");
+			String a=split[0].toLowerCase();
+			String b=split.length>1 ? split[1] : null;
+			if(b!=null){binner.parse(arg, a, b);}
+		}
+	}
+	
 	/** Ensure parameter ranges are within bounds and required parameters are set */
 	private boolean validateParams(){
 //		assert(minfoo>0 && minfoo<=maxfoo) : minfoo+", "+maxfoo;
@@ -200,42 +274,190 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 		Timer t2=new Timer();
 		
 		contigList=loader.loadData();
-
-		@SuppressWarnings("unchecked")
-		ArrayList<? extends Bin> bins=(ArrayList<Contig>) contigList.clone();
-		if(clusterByTaxid) {
-			bins=binner.clusterByTaxid(bins);
+		assert(isValid(contigList, false));
+		if(covOut!=null) {DataLoader.writeCov(covOut, contigList, loader.numDepths, outstream);}
+//		System.err.println("Thresholds:");
+//		binner.printThresholds();
+		binner.setSamples(loader.numDepths, strictnessMult);
+//		System.err.println("Thresholds:");
+//		binner.printThresholds();
+		reprocessArgs();
+//		System.err.println("Thresholds:");
+//		binner.printThresholds();
+//		assert(false);
+		
+		IntLongHashMap sizeMap=loader.sizeMap;
+//		@SuppressWarnings("unchecked")
+//		ArrayList<? extends Bin> bins=(ArrayList<Contig>) contigList.clone();
+//		if(clusterByTaxid) {
+//			bins=binner.clusterByTaxid(bins);
+//		}
+		
+		if(vectorOut!=null) {
+			Oracle.bsw=ByteStreamWriter.makeBSW(vectorOut, overwrite, append, true);
+//			Oracle.bsw.println(Oracle.header());
 		}
 		
-		ArrayList<Cluster> clusters;
-		if(clusterByTetramer) {
-			clusters=binner.makeClusters(bins);
-		}else {
-			clusters=new ArrayList<Cluster>(bins.size());
-			for(Bin b : bins) {clusters.add(b.toCluster(b.id()));}
+		Timer ct=new Timer(outstream);
+		ArrayList<Bin> binList=null;
+		if(followEdge1Passes>0 && loader.makePairGraph) {
+			t2.start();
+			outstream.println("Following Edges:");
+			assert(isValid(contigList, false));
+			int sum=0;
+			Oracle oracle=new Oracle(edgeStringency1, 2);
+			long merges=binner.followEdges(contigList, contigList, oracle);
+			sum+=merges;
+			for(int i=1; i<followEdge1Passes && merges>0; i++) {
+				binList=Binner.toBinList(contigList);
+				merges=binner.followEdges(contigList, binList, oracle);
+				sum+=merges;
+			}
+			assert(binList==null ? isValid(contigList, true) : isValid(binList, false));
+			if(merges>0) {
+				binList=Binner.toBinList(contigList);
+			}
+			t2.stop("Merged "+sum+" bins:   ");
+			fastComparisonsEdge+=oracle.fastComparisons;
+			slowComparisonsEdge+=oracle.slowComparisons;
 		}
 		
+		long fastComp=0, slowComp=0;
+		BinMap binMap=binner.makeBinMap(contigList, binList);
+		assert(binMap.isValid());
+		binList=null;
+		fastComparisonsCreate+=(binner.fastComparisons.get()-fastComp);
+		slowComparisonsCreate+=(binner.slowComparisons.get()-slowComp);
+		
 		if(refineClusters) {
-			clusters=binner.refineClusters(clusters);
+			fastComp=binner.fastComparisons.get();
+			slowComp=binner.slowComparisons.get();
+			binner.refineBinMap(binMap);
+			assert(binMap.isValid());
+			fastComparisonsRefine+=(binner.fastComparisons.get()-fastComp);
+			slowComparisonsRefine+=(binner.slowComparisons.get()-slowComp);
 		}
+		
+		if(followEdge2Passes>0 && loader.makePairGraph) {
+			fastComp=(binner.fastComparisons.get());
+			slowComp=(binner.slowComparisons.get());
+			t2.start();
+			outstream.println("Following Edges:");
+			long merges=100, sum=0;
+			assert(isValid(contigList, true));
+			Oracle oracle=new Oracle(edgeStringency2, 2);
+			for(int i=0; i<followEdge2Passes && merges>1; i++) {
+//				assert(isValid(contigList, true));
+				binList=Binner.toBinList(contigList);
+//				assert(isValid(binList, false));
+				merges=binner.followEdges(contigList, binList, oracle);
+				sum+=merges;
+			}
+			t2.stop("Merged "+sum+" bins:   ");
+			if(sum>0) {
+				binMap.clear(true);
+				binList=Binner.toBinList(contigList);
+				assert(isValid(binList, false));
+				binMap.addAll(binList, Binner.minSizeToMerge);
+			}
+			fastComparisonsEdge+=(binner.fastComparisons.get()-fastComp);
+			slowComparisonsEdge+=(binner.slowComparisons.get()-slowComp);
+		}
+		
+		if(processResidue) { 
+			fastComp=(binner.fastComparisons.get());
+			slowComp=(binner.slowComparisons.get());
+			int removedResidue=binner.processResidue(binMap, Binner.residueStringency, 
+					TaxTree.SPECIES, true, true, binner.residueRange);
+			fastComparisonsResidue+=(binner.fastComparisons.get()-fastComp);
+			slowComparisonsResidue+=(binner.slowComparisons.get()-slowComp);
+			if(removedResidue>1) {
+				fastComp=binner.fastComparisons.get();
+				slowComp=binner.slowComparisons.get();
+				//it would be best to have a bigger compare size limit for residue prior to this
+				//Possibly do this in 2 passes, with range +0 and +2
+				int removedThisPhase=binner.refinePhase(binMap, 
+						"f", 0.9f, -1, true, true, binner.baseRange, binner.basePasses+0);
+				removedThisPhase=binner.refinePhase(binMap, 
+						"g", 1.0f, -1, true, true, binner.baseRange+2, binner.basePasses+1);
+			}
+			fastComparisonsRefine+=(binner.fastComparisons.get()-fastComp);
+			slowComparisonsRefine+=(binner.slowComparisons.get()-slowComp);
+		}
+		
+		if(Oracle.bsw!=null) {
+			Oracle.bsw.poisonAndWait();
+			Oracle.bsw=null;
+		}
+		
+		if(purifyClusters) {
+			t2.start();
+			outstream.println("Purifying clusters:");
+			fastComp=(binner.fastComparisons.get());
+			slowComp=(binner.slowComparisons.get());
+			int purified=binner.purify(binMap, 
+					2f, binner.baseRange+4, minBasesPerCluster, Tools.min(minBasesPerCluster/2, 20000));
+			fastComparisonsPurify+=(binner.fastComparisons.get()-fastComp);
+			slowComparisonsPurify+=(binner.slowComparisons.get()-slowComp);
+			t2.stop("Removed "+purified+" contigs:   ");
+		}
+		ct.stop();
+		
 		t2.stop();
-		for(int i=0; i<clusters.size(); i++) {clusters.get(i).id=i;}
+		ArrayList<Cluster> bins=binMap.toList(true);
+		Collections.sort(bins);
+//		assert(isValid(bins, false));
 		
 		if(sketchOutput) {
 			System.err.println("Sketching output.");
-			sketcher.sketch(clusters, true);
+			sketcher.sketch(bins, true);
 		}
 		
-		System.err.println("\nFinal clusters:");
-		for(int i=0; i<clusters.size(); i++) {
-			Cluster a=clusters.get(i);
-			if(a.size>=4000 || a.contigs.size()>1){
-				System.out.println(a.toBytes());
+		long cleanBins=0, cleanSize=0, partialCleanSize=0;
+		long contamBins=0, contamSize=0, partialContamSize=0;
+		double contamScore=0;
+		double compltScore=0;
+		long sizeOverLimit=0, binsOverLimit=0;
+		long contigsOverLimit=0, badContigsOverLimit=0;
+		IntHashMap tidBins=new IntHashMap();
+		
+//		t2.start();
+//		System.err.print("Calculating statistics.");
+		for(Bin b : bins) {
+			if(b.size()<minBasesPerCluster) {continue;}
+			if(validation) {b.calcContam(sizeMap);}
+			long contam=(int)(b.size()*b.contam);
+			compltScore+=((b.size()-contam)*b.completeness);
+			contamScore+=(b.size()*b.contam);
+			if(contam>0) {
+				contamBins++;
+				contamSize+=b.size();
+				partialContamSize+=contam;
+				partialCleanSize+=(b.size()-contam);
+			}else {
+				cleanBins++;
+				cleanSize+=b.size();
+			}
+			if(b.size()>=minBasesPerCluster) {
+				sizeOverLimit+=b.size();
+				binsOverLimit++;
+				contigsOverLimit+=b.numContigs();
+				if(b.labelTaxid>0) {tidBins.increment(b.labelTaxid);}
+				badContigsOverLimit+=b.badContigs;
 			}
-			if(a.size<1000 || i>10) {break;}
 		}
+//		t2.stopAndPrint();
 		
-		outputClusters(outPattern, clusters, minBasesPerCluster, minContigsPerCluster);
+//		System.err.println("\nFinal clusters:");
+//		for(int i=0; i<bins.size(); i++) {
+//			Bin a=bins.get(i);
+//			if(a.size()>=4000 || a.numContigs()>1){
+//				System.out.println(a.toBytes());
+//			}
+//			if(a.size()<1000 || i>=4) {break;}
+//		}
+		
+		outputClusters(outPattern, bins, minBasesPerCluster, minContigsPerCluster);
 //		
 //		//Create a read input stream
 //		final ConcurrentReadInputStream cris=makeCris(ffin1, ffin2);
@@ -263,14 +485,62 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 //		outstream.println(Tools.timeReadsBasesProcessed(t, readsProcessed, basesProcessed, 8));
 //		outstream.println(Tools.readsBasesOut(readsProcessed, basesProcessed, readsOut, basesOut, 8, false));
 
-		outstream.println("Initial Fast Comparisons:    \t"+binner.initialComparisons);
-		outstream.println("Initial Slow Comparisons:    \t"+binner.initialComparisonsSlow);
-		outstream.println("Refinement Fast Comparisons: \t"+binner.refinementComparisons);
-		outstream.println("Refinement Slow Comparisons: \t"+binner.refinementComparisonsSlow);
-		float cps=((binner.initialComparisonsSlow+binner.refinementComparisonsSlow)/(float)t2.elapsed)*1000000000;
-		outstream.println("Comparisons Per Second:      \t"+Tools.padKMB((long)cps, 0));
+		long totalComp=binner.slowComparisons.get()+slowComparisonsEdge;
+		float cps=(totalComp/(float)ct.elapsed)*1000000000;
+		outstream.println("Initial Fast Comparisons: \t"+fastComparisonsCreate);
+		outstream.println("Initial Slow Comparisons: \t"+slowComparisonsCreate);
+		outstream.println("Grid Fast Comparisons:    \t"+fastComparisonsRefine);
+		outstream.println("Grid Slow Comparisons:    \t"+slowComparisonsRefine);
+		outstream.println("Edge Fast Comparisons:    \t"+fastComparisonsEdge);
+		outstream.println("Edge Slow Comparisons:    \t"+slowComparisonsEdge);
+		outstream.println("Residue Fast Comparisons: \t"+fastComparisonsResidue);
+		outstream.println("Residue Slow Comparisons: \t"+slowComparisonsResidue);
+		outstream.println("Purify Fast Comparisons:  \t"+fastComparisonsPurify);
+		outstream.println("Purify Slow Comparisons:  \t"+slowComparisonsPurify);
+		outstream.println("Total Slow Comparisons:   \t"+totalComp+" \t"+Tools.padKMB((long)cps, 0)+" cps");
+		outstream.println();
+
+		if(validation) {
+			outstream.println(formatString("Good Merges Follow", 29, binner.goodMergesFollow, binner.badMergesFollow));
+			outstream.println(formatString("Bad Merges Follow", 29, binner.badMergesFollow, binner.goodMergesFollow));
+			outstream.println(formatString("Bad Merge Size Follow", 29, binner.badMergeSizeFollow, binner.goodMergeSizeFollow));
+			outstream.println(formatString("Good Merges Create", 29, binner.goodMergesCreate, binner.badMergesCreate));
+			outstream.println(formatString("Bad Merges Create", 29, binner.badMergesCreate, binner.goodMergesCreate));
+			outstream.println(formatString("Bad Merge Size Create", 29, binner.badMergeSizeCreate, binner.goodMergeSizeCreate));
+			outstream.println(formatString("Good Merges Refine", 29, binner.goodMergesRefine, binner.badMergesRefine));
+			outstream.println(formatString("Bad Merges Refine", 29, binner.badMergesRefine, binner.goodMergesRefine));
+			outstream.println(formatString("Bad Merge Size Refine", 29, binner.badMergeSizeRefine, binner.goodMergeSizeRefine));
+			outstream.println(formatString("Good Merges Residue", 29, binner.goodMergesResidue, binner.badMergesResidue));
+			outstream.println(formatString("Bad Merges Residue", 29, binner.badMergesResidue, binner.goodMergesResidue));
+			outstream.println(formatString("Bad Merge Size Residue", 29, binner.badMergeSizeResidue, binner.goodMergeSizeResidue));
+			outstream.println();
+			outstream.println(formatString("Clean Bins", 29, cleanBins, contamBins));
+			outstream.println(formatString("Dirty Bins", 29, contamBins, cleanBins));
+			outstream.println(formatString("Clean Bin Bases", 29, cleanSize, contamSize));
+			outstream.println(formatString("Dirty Bin Bases", 29, contamSize, cleanSize));
+			outstream.println(formatString("Tainted Bases", 29, partialCleanSize, cleanSize+contamSize-partialCleanSize));
+			outstream.println(formatString("Contam Bases", 29, partialContamSize, cleanSize+contamSize-partialContamSize));
+			outstream.println();
+		}
+
+		outstream.println("Sequence Recovery:           \t"+
+				String.format("%.3f", sizeOverLimit*100.0/loader.basesLoaded));
+		outstream.println("Contig Recovery:             \t"+
+				String.format("%.3f", contigsOverLimit*100.0/loader.contigsLoaded));
+
+		if(validation) {
+			outstream.println("Bad Contigs:                 \t"+
+					String.format("%.3f", badContigsOverLimit*100.0/contigsOverLimit));
+			outstream.println("Genomes Represented:         \t"+
+					String.format("%.3f", (tidBins.size())*100.0/sizeMap.size()));
+			outstream.println("Completeness Score:          \t"+
+					String.format("%.3f", 100*compltScore/loader.basesLoaded));
+			outstream.println("Contamination Score:         \t"+
+					String.format("%.4f", 100*contamScore/loader.basesLoaded));
+		}
+		outstream.println();
 		
-		printL90(clusters);
+		GradeBins.printL90(bins, loader.basesLoaded);
 		
 		//Throw an exception of there was an error in a thread
 		if(errorState){
@@ -278,51 +548,33 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 		}
 	}
 	
-	private void printL90(ArrayList<Cluster> list) {
-		long sum=0;
-		for(Cluster c : list) {sum+=c.size;}
-		long c99=(long)(0.99f*sum);
-		long c95=(long)(0.95f*sum);
-		long c90=(long)(0.90f*sum);
-		long c80=(long)(0.80f*sum);
-		long c50=(long)(0.50f*sum);
-		
-		
-		long prev=0, sum2=0;
-		for(int i=0; i<list.size(); i++) {
-			Cluster c=list.get(i);
-			long size=c.size;
-			prev=sum2;
-			sum2+=size;
-			int num=i+1;
-
-			if(sum2>=c99 && prev<c99) {System.err.println("L99: "+size+"\t"+"N99: "+num);}
-			if(sum2>=c95 && prev<c95) {System.err.println("L95: "+size+"\t"+"N95: "+num);}
-			if(sum2>=c90 && prev<c90) {System.err.println("L90: "+size+"\t"+"N90: "+num);}
-			if(sum2>=c80 && prev<c80) {System.err.println("L80: "+size+"\t"+"N80: "+num);}
-			if(sum2>=c50 && prev<c50) {System.err.println("L50: "+size+"\t"+"N50: "+num);}
-		}
+	static String formatString(String term, int len, long a, long b) {
+		float pct=a*100f/(a+b);
+		while(term.length()<len) {term=term+" ";}
+		String apad=Long.toString(a);
+		while(apad.length()<12) {apad+=" ";}
+		return(term+"\t"+apad+"\t"+String.format("%.3f%%", pct));
 	}
 	
-	private void outputClusters(String pattern, ArrayList<Cluster> clusters, long minBases, int minContigs) {
+	private void outputClusters(String pattern, ArrayList<? extends Bin> clusters, long minBases, int minContigs) {
 //		if(pattern==null) {return;}
 		if(pattern!=null) {outstream.println("Writing clusters to "+pattern);}
-		
+		long sizeOverLimit=0;
 		if(pattern!=null && pattern.indexOf('%')>=0) {
 			final ByteStreamWriter chaff=ByteStreamWriter.makeBSW(pattern.replace("%", "chaff"), overwrite, append, true);
 			final ByteBuilder bb=new ByteBuilder(8192);
 			for(int i=0; i<clusters.size(); i++) {
-				Cluster a=clusters.get(i);
-				if(a.size>=minBases || a.contigs.size()>=minContigs) {
+				Bin a=clusters.get(i);
+				if(a.size()>=minBases && a.numContigs()>=minContigs) {
 					String fname=pattern.replace("%", Integer.toString(i));
 					final ByteStreamWriter bsw=ByteStreamWriter.makeBSW(fname, overwrite, append, true);
-					printCluster(a, bsw, bb, -1);
+					printBin(a, bsw, bb, -1);
 					bsw.poison();
 					clustersWritten++;
-					contigsWritten+=a.contigs.size();
-					basesWritten+=a.size;
+					contigsWritten+=a.numContigs();
+					basesWritten+=a.size();
 				}else {
-					printCluster(a, chaff, bb, i+1);
+					printBin(a, chaff, bb, i+1);
 				}
 			}
 			chaff.poisonAndWait();
@@ -330,11 +582,11 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 			final ByteBuilder bb=new ByteBuilder(8192);
 			final ByteStreamWriter bsw=ByteStreamWriter.makeBSW(pattern, overwrite, append, true);
 			for(int i=0; i<clusters.size(); i++) {
-				Cluster a=clusters.get(i);
-				printCluster(a, bsw, bb, i+1);
+				Bin a=clusters.get(i);
+				printBin(a, bsw, bb, i+1);
 				clustersWritten++;
-				contigsWritten+=a.contigs.size();
-				basesWritten+=a.size;
+				contigsWritten+=a.numContigs();
+				basesWritten+=a.size();
 			}
 			if(bsw!=null) {bsw.poisonAndWait();}
 		}
@@ -348,6 +600,11 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 				Tools.padLeft(basesWritten, 10)+"\t"+Tools.format("%.2f%%", bpct));
 	}
 	
+	private void printBin(Bin a, ByteStreamWriter bsw, ByteBuilder bb, int id) {
+		if(a.isCluster()) {printCluster((Cluster)a, bsw, bb, id);}
+		else {printContig((Contig)a, bsw, bb, id);}
+	}
+	
 	private void printCluster(Cluster a, ByteStreamWriter bsw, ByteBuilder bb, int id) {
 		ArrayList<Contig> contigs=a.contigs;
 		Collections.sort(contigs);
@@ -362,6 +619,12 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 		bb.clear();
 	}
 	
+	private void printContig(Contig c, ByteStreamWriter bsw, ByteBuilder bb, int id) {
+		c.appendTo(bb, id);
+		if(bsw!=null && !bb.isEmpty()) {bsw.print(bb);}
+		bb.clear();
+	}
+	
 //	private ConcurrentReadInputStream makeCris(FileFormat ff1, FileFormat ff2){
 //		ConcurrentReadInputStream cris=ConcurrentReadInputStream.getReadInputStream(maxReads, true, ff1, ff2);
 //		cris.start(); //Start the stream
@@ -570,14 +833,22 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 	/** Override output file extension */
 	private String extout=null;
 	
-	private ArrayList<Contig> contigList;
-	private ArrayList<Bin> binList;
+	private String covOut=null;
+	private String vectorOut=null;
 	
-	int minBasesPerCluster=8000;
-	int minContigsPerCluster=2;
-	boolean clusterByTaxid=true;
+	private ArrayList<Contig> contigList;
+	
+	boolean clusterByTaxid=false;
 	boolean clusterByTetramer=true;
 	boolean refineClusters=true;
+	boolean processResidue=true;
+	boolean purifyClusters=true;
+	int followEdge1Passes=0;
+	int followEdge2Passes=4;
+	float edgeStringency1=0.25f;
+	float edgeStringency2=1.1f;
+	
+	float strictnessMult=1f;
 	
 	/*--------------------------------------------------------------*/
 
@@ -598,6 +869,14 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 	private DataLoader loader;
 	private Binner binner;
 	
+
+	private long fastComparisonsCreate=0, slowComparisonsCreate=0;
+	private long fastComparisonsRefine=0, slowComparisonsRefine=0;
+	private long fastComparisonsEdge=0, slowComparisonsEdge=0;
+	private long fastComparisonsResidue=0, slowComparisonsResidue=0;
+	private long fastComparisonsPurify=0, slowComparisonsPurify=0;
+	
+	
 	/*--------------------------------------------------------------*/
 	/*----------------         Final Fields         ----------------*/
 	/*--------------------------------------------------------------*/
@@ -609,13 +888,12 @@ public class QuickBin extends BinObject implements Accumulator<QuickBin.ProcessT
 	@Override
 	public final ReadWriteLock rwlock() {return rwlock;}
 	private final ReadWriteLock rwlock=new ReentrantReadWriteLock();
+	private String[] originalArgs;
 	
 	/*--------------------------------------------------------------*/
 	/*----------------        Common Fields         ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	/** Print status messages to this output stream */
-	private PrintStream outstream=System.err;
 	/** Print verbose messages */
 	public static boolean verbose=false;
 	/** True if an error was encountered */
