@@ -142,6 +142,8 @@ public class RandomReadsMG {
 				maxDepth=Float.parseFloat(b);
 			}else if(a.equals("depthvariance")){
 				depthVariance=Float.parseFloat(b);
+			}else if(a.equals("mode") || a.equals("depthmode")){
+				depthMode=Tools.find(b.toUpperCase(), modes);
 			}else if(a.equals("insert") || a.equals("avginsert")){
 				avgInsert=Float.parseFloat(b);
 			}else if(a.equals("len") || a.equals("length") || a.equals("readlen") || a.equals("readlength")){
@@ -356,6 +358,43 @@ public class RandomReadsMG {
 		return cris;
 	}
 	
+	float chooseDepthForFile(Random randy) {
+		if(depthMode==MIN4) {return depthMin4(randy);}
+		else if(depthMode==EXP) {return depthExp(randy);}
+		else if(depthMode==ROOT) {return depthRoot(randy);}
+		else if(depthMode==LINEAR) {return depthLinear(randy);}
+		else {throw new RuntimeException("Unknown mode "+depthMode);}
+	}
+	
+	float depthMin4(Random randy) {
+		float minRoot=(float)Math.sqrt(minDepth);
+		float range=(float)(Math.sqrt(maxDepth)-minRoot);
+		final float rootDepth=minRoot+(Tools.min(randy.nextFloat(), randy.nextFloat(), 
+				randy.nextFloat(), randy.nextFloat()))*range;
+		final float fileDepth=rootDepth*rootDepth;
+		return fileDepth;
+	}
+	
+	float depthLinear(Random randy) {
+		float range=(float)(maxDepth-minDepth);
+		return randy.nextFloat()*range+minDepth;
+	}
+	
+	float depthRoot(Random randy) {
+		float range=(float)maxDepth-minDepth;
+		float root=randy.nextFloat();
+		return root*root*range+minDepth;
+	}
+	
+	float depthExp(Random randy) {
+		double lambda=1/Math.sqrt(minDepth*maxDepth);
+		double depth=Tools.exponential(randy, lambda);
+		while(depth<minDepth || depth>maxDepth) {
+			depth=Tools.exponential(randy, lambda);
+		}
+		return (float)depth;
+	}
+	
 	/*--------------------------------------------------------------*/
 	/*----------------         Inner Classes        ----------------*/
 	/*--------------------------------------------------------------*/
@@ -369,9 +408,6 @@ public class RandomReadsMG {
 			ros=ros_;
 			tid=tid_;
 			nextFile=nextFile_;
-			
-			minRoot=(float)Math.sqrt(minDepth);
-			range=(float)(Math.sqrt(maxDepth)-minRoot);
 		}
 		
 		//Called by start()
@@ -400,9 +436,7 @@ public class RandomReadsMG {
 			//Grab the first ListNum of reads
 			ListNum<Read> ln=cris.nextList();
 			
-			final float rootDepth=minRoot+(Tools.min(randy.nextFloat(), randy.nextFloat(), 
-					randy.nextFloat(), randy.nextFloat()))*range;
-			final float fileDepth=rootDepth*rootDepth;
+			final float fileDepth=chooseDepthForFile(randy);
 			int taxID=TaxTree.parseHeaderStatic2(fname, tree);
 			if(taxID<0 && ln.size()>0) {
 				Read c0=ln.get(0);
@@ -542,8 +576,6 @@ public class RandomReadsMG {
 		private final ArrayList<String> files;
 		private Random randy;
 		
-		private final float minRoot;
-		private final float range;
 		private ByteBuilder bb=new ByteBuilder(128);
 	}
 	
@@ -604,9 +636,13 @@ public class RandomReadsMG {
 	private float depthVariance=0.3f;
 	private float avgInsert=300;
 	private int readlen=150;
-	private boolean paired=false;
+	private boolean paired=true;
 	private long seed=-1;
 	
+	static final String[] modes={"MIN4", "EXP", "ROOT", "LINEAR"};
+	static final int MIN4=0, EXP=1, ROOT=2, LINEAR=3;
+	int depthMode=MIN4;
+	
 	/*--------------------------------------------------------------*/
 	/*----------------         Final Fields         ----------------*/
 	/*--------------------------------------------------------------*/
