@@ -337,7 +337,7 @@ public final class FileFormat {
 		
 		if(ext==null){}
 		else if(ext.equals("fq") || ext.equals("fastq") || (comp!=null && comp.equals("fqz"))){r[0]=FASTQ;}
-		else if(isFasta(ext)){r[0]=FASTA;}
+		else if(isFastaExt(ext)){r[0]=FASTA;}
 		else if(ext.equals("bread")){r[0]=BREAD;}
 		else if(ext.equals("sam")){r[0]=SAM;}
 		else if(ext.equals("csfasta")){r[0]=CSFASTA;}
@@ -627,7 +627,7 @@ public final class FileFormat {
 		return letters;
 	}
 	
-	public static boolean isFasta(String ext){
+	public static boolean isFastaExt(String ext){
 		if(ext==null){return false;}
 		return (ext.equals("fa") || ext.equals("fasta") || ext.equals("fas") || ext.equals("fna") || ext.equals("ffn")
 			|| ext.equals("frn") || ext.equals("seq") || ext.equals("fsa") || ext.equals("faa"));
@@ -636,16 +636,16 @@ public final class FileFormat {
 	public static boolean isFastaFile(String fname){
 		if(fname==null){return false;}
 		String ext=ReadWrite.rawExtension(fname);
-		return isFasta(ext);
+		return isFastaExt(ext);
 	}
 	
 	public static boolean isPgmFile(String fname){
 		if(fname==null){return false;}
 		String ext=ReadWrite.rawExtension(fname);
-		return isPgm(ext);
+		return isPgmExt(ext);
 	}
 	
-	public static boolean isAmino(String ext){
+	public static boolean isAminoExt(String ext){
 		if(ext==null){return false;}
 		return ext.equals("faa"); //TODO: Investigate whether other extensions imply AA.
 	}
@@ -689,12 +689,12 @@ public final class FileFormat {
 //		return (s.equalsIgnoreCase("stdin") || s.equalsIgnoreCase("stdout") || s.equalsIgnoreCase("stderr"));
 //	}
 	
-	public static boolean isFastq(String ext){
+	public static boolean isFastqExt(String ext){
 		if(ext==null){return false;}
 		return (ext.equals("fq") || ext.equals("fastq"));
 	}
 	
-	public static boolean isPgm(String ext){
+	public static boolean isPgmExt(String ext){
 		if(ext==null){return false;}
 		return (ext.equals("pgm") || ext.equals("pkm"));
 	}
@@ -702,10 +702,10 @@ public final class FileFormat {
 	public static boolean isFastqFile(String fname){
 		if(fname==null){return false;}
 		String ext=ReadWrite.rawExtension(fname);
-		return isFastq(ext);
+		return isFastqExt(ext);
 	}
 	
-	public static boolean isSamOrBam(String ext){
+	public static boolean isSamOrBamExt(String ext){
 		if(ext==null){return false;}
 		return (ext.equals("sam") || ext.equals("bam"));
 	}
@@ -713,15 +713,15 @@ public final class FileFormat {
 	public static boolean isSamOrBamFile(String fname){
 		if(fname==null){return false;}
 		String ext=ReadWrite.rawExtension(fname);
-		return isSamOrBam(ext);
+		return isSamOrBamExt(ext);
 	}
 	
-	public static boolean isBam(String ext){
+	public static boolean isBamExt(String ext){
 		if(ext==null){return false;}
 		return ext.equalsIgnoreCase("bam");
 	}
 	
-	public static boolean isVcf(String ext){
+	public static boolean isVcfExt(String ext){
 		if(ext==null){return false;}
 		return ext.equalsIgnoreCase("vcf");
 	}
@@ -729,13 +729,13 @@ public final class FileFormat {
 	public static boolean isBamFile(String fname){
 		if(fname==null){return false;}
 		String ext=ReadWrite.rawExtension(fname);
-		return isBam(ext);
+		return isBamExt(ext);
 	}
 	
 	public static boolean isVcfFile(String fname){
 		if(fname==null){return false;}
 		String ext=ReadWrite.rawExtension(fname);
-		return isVcf(ext);
+		return isVcfExt(ext);
 	}
 
 	public void deleteIfPresent() {
@@ -764,7 +764,7 @@ public final class FileFormat {
 	private int rawExtensionCode(String ext, String comp) {
 		if(ext==null){return UNKNOWN;}
 		else if(ext.equals("fq") || ext.equals("fastq") || (comp!=null && comp.equals("fqz"))){return FASTQ;}
-		else if(isFasta(ext)){return FASTA;}
+		else if(isFastaExt(ext)){return FASTA;}
 		else if(ext.equals("bread")){return BREAD;}
 		else if(ext.equals("sam")){return SAM;}
 		else if(ext.equals("csfasta")){return CSFASTA;}
