@@ -1216,7 +1216,7 @@ public class SamLine implements Serializable {
 	}
 
 	public int mappedNonClippedBases() {
-		if(!mapped() || cigar==null || seq==null){return 0;}
+		if(!mapped() || cigar==null){return 0;}
 		
 		int len=0;
 		int current=0;
@@ -1809,7 +1809,7 @@ public class SamLine implements Serializable {
 		ArrayList<String> optionalTags=new ArrayList<String>(8);
 
 		if(mapped){
-			if(!r.secondary() && r.ambiguous()){optionalTags.add("XT:A:R");} //Not sure what do do for secondary alignments
+			if(!r.secondary() && r.ambiguous() && MAKE_XT_TAG){optionalTags.add("XT:A:R");} //Not sure what do do for secondary alignments
 
 //			int nm=r.length();
 //			int dels=0;
@@ -1933,6 +1933,8 @@ public class SamLine implements Serializable {
 
 			if(MAKE_IDENTITY_TAG && (perfect || r.match!=null)){optionalTags.add(makeIdentityTag(r.match, perfect));}
 
+			if(MAKE_MATEQ_TAG && r.mateMapped()){optionalTags.add("YQ:i:"+toMapq(r.mate, null));}
+
 			if(MAKE_SCORE_TAG && r.mapped()){optionalTags.add(makeScoreTag(r.mapScore));}
 
 			if(MAKE_INSERT_TAG && r2!=null){
@@ -2514,6 +2516,8 @@ public class SamLine implements Serializable {
 //		HashMap sc
 //	}
 	
+	public boolean ambiguous() {return mapped() && mapq<4;}
+	
 	/** Assumes rname is an integer. */
 	public int chrom_old(){
 		assert(false);
@@ -2655,14 +2659,22 @@ public class SamLine implements Serializable {
 	/*----------------           Fields             ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	public String mdTag(){
+	public String findTag(String prefix) {
 		if(optional==null){return null;}
 		for(String s : optional){
-			if(s.startsWith("MD:Z:")){return s;}
+			if(s.startsWith(prefix)){return s;}
 		}
 		return null;
 	}
 
+	public String mdTag(){return findTag("MD:Z:");}
+	public String mateqTag(){return findTag("YQ:i:");}
+	public int parseIntFlag(String prefix) {
+		String tag=findTag(prefix);
+		return tag==null ? Integer.MIN_VALUE : Parse.parseInt(tag, 5);
+	}
+	public int mateq() {return parseIntFlag("YQ:i:");}
+
 	public int setScafnum(ScafMap scafMap) {
 		assert(scafnum<0);
 		
@@ -2738,7 +2750,7 @@ public class SamLine implements Serializable {
 		if(NO_TAGS){return false;}
 		return MAKE_AM_TAG || MAKE_NM_TAG || MAKE_SM_TAG || MAKE_XM_TAG || MAKE_XS_TAG || MAKE_AS_TAG ||
 				MAKE_NH_TAG || MAKE_TOPHAT_TAGS || MAKE_IDENTITY_TAG || MAKE_SCORE_TAG || MAKE_STOP_TAG || MAKE_LENGTH_TAG ||
-				MAKE_CUSTOM_TAGS || MAKE_INSERT_TAG || MAKE_CORRECTNESS_TAG || MAKE_TIME_TAG || MAKE_BOUNDS_TAG;
+				MAKE_CUSTOM_TAGS || MAKE_INSERT_TAG || MAKE_CORRECTNESS_TAG || MAKE_TIME_TAG || MAKE_BOUNDS_TAG || MAKE_MATEQ_TAG;
 	}
 	
 	public static boolean makeAnyTags(){
@@ -2770,6 +2782,7 @@ public class SamLine implements Serializable {
 	public static boolean MAKE_SM_TAG=false;
 	public static boolean MAKE_XM_TAG=false;
 	public static boolean MAKE_XS_TAG=false;
+	public static boolean MAKE_XT_TAG=true;
 	public static boolean MAKE_AS_TAG=false; //TODO: Alignment score from aligner
 	public static boolean MAKE_NH_TAG=false;
 	public static boolean MAKE_TOPHAT_TAGS=false;
@@ -2783,6 +2796,7 @@ public class SamLine implements Serializable {
 	public static boolean MAKE_CORRECTNESS_TAG=false;
 	public static boolean MAKE_TIME_TAG=false;
 	public static boolean MAKE_BOUNDS_TAG=false;
+	public static boolean MAKE_MATEQ_TAG=false;
 	
 	public static boolean PENALIZE_AMBIG=true;
 	public static boolean CONVERT_CIGAR_TO_MATCH=true;
